@@ -184,7 +184,7 @@ gkd_dbus_singleton_acquire (gboolean *acquired)
 	/* Try and grab our name */
 	if (!acquired_asked) {
 
-#ifdef WITH_TESTS
+#ifdef WITH_TESTABLE
 		service = g_getenv ("GNOME_KEYRING_TEST_SERVICE");
 		if (service && service[0])
 			flags = DBUS_NAME_FLAG_ALLOW_REPLACEMENT | DBUS_NAME_FLAG_REPLACE_EXISTING;
