@@ -136,8 +136,13 @@ add_certificate_for_data (GkmRootsModule *self, const guchar *data,
 }
 
 static void
-parsed_pem_block (GQuark type, const guchar *data, gsize n_data,
-                  GHashTable *headers, gpointer user_data)
+parsed_pem_block (GQuark type,
+                  const guchar *data,
+                  gsize n_data,
+                  const gchar *outer,
+                  gsize n_outer,
+                  GHashTable *headers,
+                  gpointer user_data)
 {
 	static GQuark PEM_CERTIFICATE;
 	static volatile gsize quarks_inited = 0;
@@ -181,7 +186,6 @@ file_load (GkmFileTracker *tracker, const gchar *path, GkmRootsModule *self)
 	GList *objects, *l;
 	GError *error = NULL;
 	gsize n_data;
-	guint num;
 
 	manager = gkm_module_get_manager (GKM_MODULE (self));
 	g_return_if_fail (manager);
@@ -206,7 +210,7 @@ file_load (GkmFileTracker *tracker, const gchar *path, GkmRootsModule *self)
 	g_list_free (objects);
 
 	/* Try and parse the PEM */
-	num = egg_openssl_pem_parse (data, n_data, parsed_pem_block, &ctx);
+	egg_openssl_pem_parse (data, n_data, parsed_pem_block, &ctx);
 
 	/* If no PEM data, try to parse directly as DER  */
 	if (ctx.count == 0) {
